package utils

import (
	"fmt"
	"net/http"
	"net/url"
	"os"

	"github.com/go-resty/resty/v2"
)

// https://github.com/FlareSolverr/FlareSolverr#-requestget
type request struct {
	Cmd        string `json:"cmd"`
	Url        string `json:"url"`
	MaxTimeout int    `json:"maxTimeout"`
}

type solution struct {
	Status   int               `json:"status"`
	Response []byte            `json:"response"`
	Headers  map[string]string `json:"headers"`
}

type response struct {
	Solution solution `json:"solution"`
}

func GET(target string) ([]byte, int, http.Header, error) {
	var (
		client *resty.Client = resty.New()
		frurl  string        = ""
	)

	if frurl = os.Getenv("FLARERESOLVER"); frurl == "" {
		if res, err := client.R().Get(target); err != nil {
			return nil, 0, nil, err
		} else {
			return res.Body(), res.StatusCode(), res.Header(), nil
		}
	}

	frurl, _ = url.JoinPath(frurl, "/v1")

	res, err := client.R().
		SetBody(request{
			Cmd:        "request.get",
			Url:        target,
			MaxTimeout: 15000,
		}).
		SetResult(&response{}).
		Post(frurl)

	if err != nil {
		return nil, 0, nil, err
	}

	if res.StatusCode() != 200 {
		return nil, 0, nil, fmt.Errorf("flareresolver failure")
	}

	response := res.Result().(*response)
	headers := http.Header{}

	for k, v := range response.Solution.Headers {
		headers.Add(k, v)
	}

	return response.Solution.Response, response.Solution.Status, headers, nil
}
